library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity memblocco is
port( rst,clk,nw,rw: in std_logic;
      datain: in std_logic_vector(15 downto 0);
      dataout: out std_logic_vector(15 downto 0);
      ready: out std_logic );
end memblocco;

architecture beh of memblocco is
type stati is (idle,readnum,readaddr,memaccess);
type memoria is array( 0 to 31) of std_logic_vector(15 downto 0);
signal cs : stati;
signal number, current,last: std_logic_vector(4 downto 0);
signal mem: memoria;
signal num_en,addr_en,mem_en: std_logic;
begin

   process(clk,rst)
     begin
      if rst='1'
       then  cs <= idle;
      elsif clk'event and clk='0' then
         case cs is
           when idle => if nw ='1' then cs <= readnum;
                        else cs <= idle;
                        end if;
           when readnum => cs <= readaddr;
           when readaddr => cs <= memaccess;
           when memaccess => if current = last then 
                                cs <= idle;
                             else cs <= memaccess;
                        end if;
           when others => cs <= idle;
         end case;
       end if;
   end process;
   
   process(cs)
    begin
      case cs is
        when readnum =>  num_en<= '1';
        		 addr_en<= '0';
        		 mem_en <= '0';
        when readaddr => addr_en<= '1';
                         num_en<= '0';
                         mem_en <= '0';
        when memaccess => mem_en <= '1';
        		     addr_en<= '0';
                             num_en<= '0';
        when others =>   addr_en<= '0';
                         num_en<= '0';
                         mem_en <= '0';
     end case;
   end process;
   
   
   process(current,mem_en,rw)
    begin
      if mem_en ='1' then
         if rw= '1' then dataout <= mem(conv_integer(current)) after 3 ns;
         else mem(conv_integer(current)) <= datain after 3 ns;
              dataout <= (others => 'Z');
         end if;
      else dataout <= (others => 'Z');
      end if;
    end process;
    
    process(clk)
    begin
     if clk'event and clk='0' then
     
      if num_en = '1'then
       number <= datain(4 downto 0);
      end if;
      
      if addr_en = '1' then 
         last <= datain(4 downto 0) +number -1;
      end if;  
    
      if addr_en = '1' then current <= datain(4 downto 0);
      elsif mem_en = '1' and current < last then current <= current + 1;
      end if;
    
     if mem_en = '1' and  current = last then
        ready <= '1';
      else ready <= '0';
      end if;
    end if;   
   end process;
    
 end beh;
     

